'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _htmlTags = require('html-tags');

var _htmlTags2 = _interopRequireDefault(_htmlTags);

var _stringifier = require('postcss/lib/stringifier');

var _stringifier2 = _interopRequireDefault(_stringifier);

var _postcssSelectorParser = require('postcss-selector-parser');

var _postcssSelectorParser2 = _interopRequireDefault(_postcssSelectorParser);

var _postcssValueParser = require('postcss-value-parser');

var _postcssValueParser2 = _interopRequireDefault(_postcssValueParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var mediaTypes = ['all', 'aural', 'braille', 'embossed', 'handheld', 'print', 'projection', 'screen', 'tty', 'tv'];

var vendors = ['-webkit-', '-moz-', '-ms-', '-o-'];

var span = function span(text, element) {
    return '<span class="midas__' + element + '">' + text + '</span>';
};

var MidasStringifier = function (_Stringifier) {
    _inherits(MidasStringifier, _Stringifier);

    MidasStringifier.prototype.atrule = function atrule(node, semicolon) {
        var name = span('@' + node.name, 'at-rule-name');
        var params = node.params ? this.rawValue(node, 'params') : '';

        if (params) {
            var parsed = (0, _postcssValueParser2.default)(this.rawValue(node, 'params'));
            parsed.nodes.forEach(function (child) {
                if (child.type === 'string') {
                    var quote = child.quote;

                    child.value = span(quote + child.value + quote, 'string');
                    child.quote = null;
                    return;
                }
                if (~mediaTypes.indexOf(child.value) && node.name === 'media') {
                    child.value = span(child.value, 'at-rule-keyword');
                    return;
                }
                if (child.type === 'function' && !child.value) {
                    child.nodes.forEach(function (n, i) {
                        if (n.type === 'div' && n.value === ':') {
                            n.value = span(':', 'colon');
                        }
                        if (n.type === 'word') {
                            var number = (0, _postcssValueParser.unit)(n.value);
                            if (number) {
                                n.value = span(n.value, 'number');
                                return;
                            }
                            if (child.nodes[i + 1] && child.nodes[i + 1].value === ':') {
                                n.value = span(n.value, 'property');
                            }
                        }
                    });
                    child.value = span('(', 'parenthesis') + (0, _postcssValueParser.stringify)(child.nodes) + span(')', 'parenthesis');
                    child.nodes = null;
                }
            });
            params = (0, _postcssValueParser.stringify)(parsed);
        }

        if (typeof node.raws.afterName !== 'undefined') {
            name += node.raws.afterName;
        } else if (params) {
            name += ' ';
        }

        if (node.nodes) {
            this.block(node, span(name + params, 'at-rule'));
        } else {
            var semi = semicolon ? span(';', 'semicolon') : '';
            var end = (node.raws.between || '') + semi;
            this.builder(span(name + params + end, 'at-rule'), node);
        }
    };

    MidasStringifier.prototype.block = function block(node, start) {
        var between = this.raw(node, 'between', 'beforeOpen');
        this.builder('' + start + between + span('{', 'brace'), node, 'start');

        var after = void 0;
        if (node.nodes && node.nodes.length) {
            this.body(node);
            after = this.raw(node, 'after');
        } else {
            after = this.raw(node, 'after', 'emptyBody');
        }

        if (after) {
            this.builder(after);
        }
        this.builder(span('}', 'brace'), node, 'end');
    };

    function MidasStringifier(builder) {
        var opts = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

        _classCallCheck(this, MidasStringifier);

        var _this = _possibleConstructorReturn(this, _Stringifier.call(this, builder));

        _this.opts = _extends({
            wrap: true
        }, opts);
        return _this;
    }

    MidasStringifier.prototype.comment = function comment(node) {
        var left = this.raw(node, 'left', 'commentLeft');
        var right = this.raw(node, 'right', 'commentRight');
        this.builder(span('/*' + left + node.text + right + '*/', 'comment'), node);
    };

    MidasStringifier.prototype.decl = function decl(node, semicolon) {
        var between = span(this.raw(node, 'between', 'colon'), 'colon');
        var parsed = (0, _postcssValueParser2.default)(this.rawValue(node, 'value'));
        var transform = function transform(n) {
            if (n.type === 'string') {
                var quote = n.quote;

                n.value = span(quote + n.value + quote, 'string');
                n.quote = null;
                return;
            }
            if (n.type === 'word') {
                var number = (0, _postcssValueParser.unit)(n.value);
                if (number) {
                    n.value = span(n.value, 'number');
                    return;
                }
                if (!n.value.indexOf('#')) {
                    var val = span(n.value.slice(1), 'hex-value');
                    n.value = span('#' + val, 'hex-color');
                    return;
                }
                n.value = span(n.value, 'word');
                return;
            }
            if (n.type === 'function') {
                n.nodes.forEach(transform);
                n.value = span(span(n.value, 'function-name') + span('(' + n.before, 'parenthesis') + (0, _postcssValueParser.stringify)(n.nodes) + span(n.after + ')', 'parenthesis'), 'function');
                n.nodes = null;
            }
        };
        parsed.nodes.forEach(transform);
        vendors.some(function (vendor) {
            if (~node.prop.indexOf(vendor)) {
                var prefix = span(vendor, 'vendor-prefix');
                node.prop = node.prop.replace(vendor, prefix);
            }
        });

        var prop = span(node.prop, 'property');
        var string = prop + between + span(String(parsed), 'value');

        if (node.important) {
            string += span(node.raws.important || ' !important', 'important');
        }

        if (semicolon) {
            string += span(';', 'semicolon');
        }
        this.builder(string, node);
    };

    MidasStringifier.prototype.root = function root(node) {
        if (this.opts.wrap) {
            this.builder('<pre class="midas"><code>');
            _Stringifier.prototype.root.call(this, node);
            this.builder('</code></pre>');
        } else {
            _Stringifier.prototype.root.call(this, node);
        }
    };

    MidasStringifier.prototype.rule = function rule(node) {
        if (node.selector) {
            this.selector(node);
        }
        this.block(node, span(this.rawValue(node, 'selector'), 'selector'));
    };

    MidasStringifier.prototype.selector = function selector(node) {
        var transform = function transform(selectors) {
            selectors.walk(function (sel) {
                if (sel.type === 'tag' && ~_htmlTags2.default.indexOf(sel.value)) {
                    sel.value = span(sel.value, 'tag');
                }
                if (sel.type === 'attribute') {
                    sel.toString = function () {
                        var selector = [this.spaces.before, span('[', 'attribute-brace'), this.ns, span(this.attribute, 'attribute-name')];

                        if (this.operator) {
                            var op = span(this.operator, 'attribute-operator');
                            selector.push(op);
                        }
                        if (this.value) {
                            selector.push(span(this.value, 'attribute-value'));
                        }
                        var ai = 'attribute-insensitive';
                        if (this.raws.insensitive) {
                            var ci = span(this.raws.insensitive, ai);
                            selector.push(ci);
                        } else if (this.insensitive) {
                            selector.push(span(' i'), ai);
                        }
                        selector.push(span(']', 'attribute-brace'));
                        var str = selector.concat(this.spaces.after).join('');
                        return span(str, 'attribute');
                    };
                }
                if (sel.type === 'pseudo') {
                    sel.toString = function () {
                        var params = this.length ? span('(', 'parenthesis') + this.map(String).join(',') + span(')', 'parenthesis') : '';

                        return [this.spaces.before, span(String(this.value), 'pseudo'), params, this.spaces.after].join('');
                    };
                }
                if (sel.type === 'class') {
                    sel.toString = function () {
                        return [this.spaces.before, this.ns, span(String('.' + this.value), 'class'), this.spaces.after].join('');
                    };
                }
                if (sel.type === 'id') {
                    sel.toString = function () {
                        return [this.spaces.before, this.ns, span(String('#' + this.value), 'id'), this.spaces.after].join('');
                    };
                }
                if (sel.type === 'universal') {
                    sel.value = span(sel.value, 'universal');
                }
                if (sel.type === 'combinator') {
                    sel.value = span(sel.value, 'combinator');
                }
            });
        };

        node.selector = (0, _postcssSelectorParser2.default)(transform).process(node.selector).result;
    };

    return MidasStringifier;
}(_stringifier2.default);

exports.default = MidasStringifier;
module.exports = exports['default'];